set time 0

################################################################
# proc buildGUI {height}--
#    Create the application GUI
# Arguments
#   height	: The initial height of the lander
# 
# Results
#   Creates scale widget, attached to "burn" variable.
#   Creates button to initiate simulation.
#   Creates a canvas and displays an empty X/Y graph

proc buildGUI {height} {
    global ready planet gravity ht destination

    set ht $height
    
    wm title . "Rocket Lander"
    set ready 0

    set w [labelframe .lfPlanet -text "Planet" -padx 2 ]
    grid .lfPlanet -row 3 -column 1 
    label .lfPlanet.l1 -text "Destination:  $destination"
    label .lfPlanet.l2 -text "Acceleration:  $gravity"
    grid .lfPlanet.l1 -row 1 -column 1
    grid .lfPlanet.l2 -row 2 -column 1

    foreach txt {"Speed" "Fuel" "Height" } var {speed fuel ht} \
        column {3 4 5} row {3 3 3} {
      set w [labelframe .lf$var -text $txt]
      grid $w -row $row -column $column
      set w [label $w.l-$var -textvariable $var -width 8]
      grid $w
    }

    scale .s -digits 3 -from 0 -to 10 \
        -resolution .1 -showvalue true -length 400 \
	-label "Fuel per second to burn"  \
	-variable burn -orient horizontal
    grid .s -row 2 -column 1  -sticky ew -columnspan 5
}

################################################################
# proc showState {i speed fuel ht}--
#    Displays an oval on the canvas scaled for speed and weight and 
#    located for height and  time.
# Arguments
#   i		The time in seconds
#   speed	The speed in meters/second
#   fuel	The mass of fuel remaining
#   ht		The height of the lander in meters
# Results
#   Updates the display.
#   Creates a binding on the new oval to display details.

proc + {a b} {
    return [expr $a + $b]
}    
proc - {a b} {
    return [expr $a - $b]
}    

proc showState {} {
   global time speed fuel ht burn
return
   # Scale values as necessary
   set x [expr {$time * 25 + 40}]
   set y [expr {460 - ($ht * .044)}] 
   set y2 [expr {$y - ($speed / 10)}]
   set wid [expr {2 + $fuel/150.0}]
   set tall [expr abs($speed)/5.0]
   set y [expr $y - $tall]
   
   # Create the oval

    set item tag-$time
    
    if {$speed > 0} {
      set fill white
    } else {
      set fill black
    }

    if {$speed > 0} {
      set fill gray75
    } else {
      set fill black
    }

    .c create polygon $x $y \
        [+ $x $wid] [+ $y 10 ] \
	[+ $x $wid] [+ $y $tall] \
	[+ $x (5+$wid)] [+ $y ($tall+4)] \
	[+ $x (5+$wid)] [+ $y ($tall+9)] \
 	[+ $x ($wid-5)] [+ $y $tall ] \
 	[- $x ($wid-5)] [+ $y $tall ] \
	[- $x (5+$wid)] [+ $y ($tall+9)] \
 	[- $x (5+$wid)] [+ $y ($tall+4)] \
	[- $x $wid] [+ $y $tall ] \
        [- $x $wid] [+ $y 10] \
	-fill $fill -outline black -tag $item

#	-smooth 1 -splinesteps 20 \

    if {$burn > 0} {
       .c create polygon [+ $x 3] [+ $y (3+$tall)] \
          $x [+ $y ($tall+$burn*3)] \
 	  [- $x 3] [+ $y (3+$tall)]  -outline black -tag $item
    }
    
   set ypos [expr {$y2 - 30}]
   .c bind $item <Button-1> "showDetails $x $ypos $speed $fuel $ht $burn"

   incr x -500
   .c xview moveto [expr {double($x) / 4000} ]
}

################################################################
# proc showDetails {x y speed fuel ht}--
#    Displays details associated with a point on the graph
# Arguments
#   x, y	The location at which to display the text
#   speed	The speed of the lander in meters/second
#   fuel	The mass of remaining fuel
#   ht		The height of the lander in meters
# Results
#   
# 
proc showDetails {x y speed fuel ht burn} {
   .c delete info
   set s [format %7.2f $speed] 
   set h [format %7.2f $ht ]
   .c create text $x $y -tags info \
       -text "Speed: $s Height: $h Fuel: $fuel Burn: $burn" \
       -anchor w
}
